--[[=============================================================================
    Copyright 2015 Control4 Corporation. All Rights Reserved.
===============================================================================]]

-- This macro is utilized to identify the version string of the driver template version used.
if (TEMPLATE_VERSION ~= nil) then
	TEMPLATE_VERSION.device_specific_commands = "2015.03.31"
end

--[[=============================================================================
    ExecuteCommand Code

    Define any functions for device specific commands (EX_CMD.<command>)
    received from ExecuteCommand that need to be handled by the driver.
===============================================================================]]
--function EX_CMD.NEW_COMMAND(tParams)
--	LogTrace("EX_CMD.NEW_COMMAND")
--	LogTrace(tParams)
--end

--[[
	Device Command: ZonePower
	Parameters:
		Power: Power state: values are "ON" and "OFF"
		Zone: zone id (1 based, must be corrected for zero based connection ids)
--]]

function EX_CMD.AudioMode(tParams)
    LogTrace("EX_CMD.AudioMode(), Mode = " .. tParams["Mode"])
    
    local mode = tParams["Mode"]
    local command = "AUDMODE(" .. tAudioModeMap[mode] .. ")"
    PackAndQueueCommand("Audio Mode", command)
end

function EX_CMD.DisplayBrightness(tParams)
    LogTrace("EX_CMD.DisplayBrightness(), Level = " .. tParams["Level"])
    
    local level = tParams["Level"]
    local X = 0
    if (level == "100%") then
	   X = 0
    elseif (level == "75%") then
	   X = 1
    elseif (level == "50%") then
	   X = 2
    elseif (level == "25%") then
	   X = 3
    end
    local command = "DIM(" .. X .. ")"
    PackAndQueueCommand("Display Brightness", command)
end

function EX_CMD.FocusPosition(tParams)
    LogTrace("EX_CMD.FocusPosition(), Mode = " .. tParams["Mode"])
    
    local mode = tParams["Mode"]
    local command = "RP(" .. tFocusPositionMap[mode] .. ")"
    PackAndQueueCommand("Focus Position", command)
end

function EX_CMD.HDMIMainOut(tParams)
    LogTrace("EX_CMD.HDMIMainOut(), Output = " .. tParams["Output"])
    
    local Output = tParams["Output"]
    local X = 0
    if (Output == "None") then
	   X = 0
    elseif (Output == "HDMI Out 1") then
	   X = 1
    elseif (Output == "HDMI Out 2") then
	   X = 2
    elseif (Output == "HDMI Out 3") then
	   X = 3
    elseif (Output == "HDMI Out 4") then
	   X = 4
    elseif (Output == "HDBT Out") then
	   X = 5
    elseif (Output == "Video Wall") then
	   X = 7
    end
   
    local command = "HDMIMAINOUT(" .. X .. ")"
    PackAndQueueCommand("HDMI Main Out", command)
end

function EX_CMD.HDMIOut(tParams)
    LogTrace("EX_CMD.HDMIOut(), Output = " .. tParams["Output"] .. ", Input = " .. tParams["Input"])
    
    
    local Input = tParams["Input"]
    local X = 0
    if (Input == "None") then
	   X = 0
    elseif (Input == "HDMI 1") then
	   X = 1
    elseif (Input == "HDMI 2") then
	   X = 2
    elseif (Input == "HDMI 3") then
	   X = 3
    elseif (Input == "HDMI 4") then
	   X = 4
    elseif (Input == "HDMI 5") then
	   X = 5
    elseif (Input == "HDMI 6") then
	   X = 6
    elseif (Input == "HDMI 7") then
	   X = 7
    elseif (Input == "HDMI 8") then
	   X = 8
    elseif (Input == "Internal") then
	   X = 9
    end
    
    local Output = tParams["Output"]
    local cmd = ""
    if (Output == "HDMI Out 1") then
	   cmd = "HDMIOUT1"
    elseif (Output == "HDMI Out 2") then
	   cmd = "HDMIOUT2"
    elseif (Output == "HDMI Out 3") then
	   cmd = "HDMIOUT3"
    elseif (Output == "HDMI Out 4") then
	   cmd = "HDMIOUT4"
    elseif (Output == "HDBT Out") then
	   cmd = "HDMIOUT5"
    end
   
    local command = cmd .. "(" .. X .. ")"
    PackAndQueueCommand("HDMI Main Out", command)
end

function EX_CMD.Loudness(tParams)
    LogTrace("EX_CMD.Loudness(), Mode = " .. tParams["Mode"])
    
    local mode = tParams["Mode"]
    local X = 0
    if (mode == "ON") then
	   X = 1
    elseif (mode == "OFF") then
	   X = 0
    end
    
    local command = "LOUDNESS(" .. X .. ")"
    PackAndQueueCommand("Loudness", command)
end

function EX_CMD.MaxVolume(tParams)
    LogTrace("EX_CMD.MaxVolume(), Level = " .. tParams["Level"])
    
    local level = tParams["Level"]
    local command = "MAXVOL(" .. level .. ")"
    PackAndQueueCommand("MaxVolume", command)
end

function EX_CMD.MultiviewLayout(tParams)
    LogTrace("EX_CMD.MultiviewLayout(), Layout = " .. tParams["Layout"])

    local layout = tParams["Layout"]
    local X = 0
    if (layout == "OFF") then
	   X = 0
    elseif (layout == "Layout 1") then
	   X = 1
    elseif (layout == "Layout 2") then
	   X = 2
    elseif (layout == "Layout 3") then
	   X = 3
    elseif (layout == "Layout 4") then
	   X = 4
    elseif (layout == "Layout 5") then
	   X = 5
    end
    
    local command = "MVIEW(" .. X .. ")"
    PackAndQueueCommand("Multiview Layout", command)
end

function EX_CMD.ActiveMultiviewWindow(tParams)
    LogTrace("EX_CMD.ActiveMultiviewWindow(), Window = " .. tParams["Window"])

    local window = tParams["Window"]
    local X = 1
    if (window == "Window 1") then
	   X = 1
    elseif (window == "Window 2") then
	   X = 2
    elseif (window == "Window 3") then
	   X = 3
    elseif (window == "Window 4") then
	   X = 4
    end
    
    local command = "MVIEWACTIVE(" .. X .. ")"
    PackAndQueueCommand("Active Multiview Window", command)
end

function EX_CMD.PulseCommand(tParams)
    LogTrace("EX_CMD.PulseCommand(), Selection  = " .. tParams["Selection"] .. ", Direction = " .. tParams["Direction"])
    
    local tCmd = {
	   ["NEXT"] = "+",
	   ["PREVIOUS"] = "-",
    }
	
    local tSelection = {
	   ["Audio Mode"] = "AUDMODE",
	   ["Focus Position"] = "RP",
	   ["Voicing"] = "VOI",
    }
    
    if (tSelection[tParams["Selection"]] == nil or tCmd[tParams["Direction"]] == nil) then
	   LogWarn("Exiting Pulse Command, parmaters not valid!!!")
	   return
    end
	
    local command = tSelection[tParams["Selection"]] .. tCmd[tParams["Direction"]]
    PackAndQueueCommand("Pulse " .. tParams["Selection"] .. " " .. tParams["Direction"], command)
end

function EX_CMD.TrimLipsync(tParams)
    LogTrace("EX_CMD.TrimLipsync(), Direction = " .. tParams["Direction"])
    
    local tCmd = {
	   ["UP"] = "+",
	   ["DOWN"] = "-",
    }
	
    local command = "LIPSYNC" .. tCmd[tParams["Direction"]]
    PackAndQueueCommand("Trim Lipsync " .. tParams["Direction"], command)
end

function EX_CMD.TrimLevel(tParams)
	LogTrace("EX_CMD.Trim()")
	LogTrace(tParams)
	
	local tCmd = {
		["UP"] = "+",
		["DOWN"] = "-",
     }
	
	local tSelection = {
		["BASS"] = "TRIMBASS",
		["CENTER"] = "TRIMCENTER",
		["HEIGHT"] = "TRIMHEIGHT",
		["LFE"] = "TRIMLFE",
		["SURROUND"] = "TRIMSURRS",
		["TREBLE"] = "TRIMTREB",
	}
	
	local command = tSelection[tParams["Selection"]] .. tCmd[tParams["Direction"]]
     PackAndQueueCommand("Trim " .. tParams["Selection"] .. " " .. tParams["Direction"], command)
end

function EX_CMD.Voicing(tParams)
    LogTrace("EX_CMD.Voicing(), Name = " .. tParams["Name"])
    
    local name = tParams["Name"]
    local command = "VOI(" .. tVoicingMap[name] .. ")"
    PackAndQueueCommand("Voicing", command)
end

function EX_CMD.ZonePower(tParams)
	LogTrace("EX_CMD.ZonePower()")
	LogTrace(tParams)
	if(tParams["Power"] == "ON") then
		local output = tonumber(tParams["Zone"]) - 1
		--we should clear gPowerState for this zone since it may be out of sync, 
		--(i.e. the user may be firing this command to correct a problem where the the unit is off when it should be on)
		gReceiverProxy._PowerState[output] =  ""		
		PRX_CMD.CONNECT_OUTPUT(RECEIVER_PROXY_BINDINGID, {OUTPUT = output})
	else
		local output = tonumber(tParams["Zone"]) - 1	
		PRX_CMD.DISCONNECT_OUTPUT(RECEIVER_PROXY_BINDINGID, {OUTPUT = output})
	end
end


-----------HELPER FUCNTIONS-----------

tAudioModeMap = {}
tFocusPositionMap = {}
tVoicingMap = {}

function GetAudioModesForProgramming()
    LogTrace('Getting Audio Modes...')
	
    local t = {}
    for j,k in pairs(tAudioModeMap) do
	   t[j] = j
    end
    local list = getSortedList(t)

    if (table.maxn(list) == 0) then 
	   LogWarn("Building Audio Modes List, close dialog window and then try again!")
	   table.insert (list, {text = "Getting list of Audio Modes from Processor. Close dialog and then retry." , value = ""})
    end
	
	--refresh the list for next launch in the event that the data has changed on the TV
    get_audio_modes()

    return list
	
end

function get_audio_modes()
    LogTrace ("get_audio_modes()")
    local command = "AUDMODEL?"
    PackAndQueueCommand("Get Audio Modes", command)
end

function GetFocusPositionsForProgramming()
    LogTrace('Getting Focus Positions...')
	
    local t = {}
    for j,k in pairs(tFocusPositionMap) do
	   t[j] = j
    end
    local list = getSortedList(t)

    if (table.maxn(list) == 0) then 
	   LogWarn("Building Focus Positions List, close dialog window and then try again!")
	   table.insert (list, {text = "Getting list of Focus Positions from Processor. Close dialog and then retry." , value = ""})
    end
	
	--refresh the list for next launch in the event that the data has changed on the TV
    get_focus_fositions()

    return list
	
end

function get_focus_fositions()
    LogTrace ("get_focus_positions()")
    local command = "RPLIST?"
    PackAndQueueCommand("Get Focus Positions", command)
end

function GetVoicingsForProgramming()
    LogTrace('Getting Voicings...')
	
    local t = {}
    for j,k in pairs(tVoicingMap) do
	   t[j] = j
    end
    local list = getSortedList(t)

    if (table.maxn(list) == 0) then 
	   LogWarn("Building Voicings List, close dialog window and then try again!")
	   table.insert (list, {text = "Getting list of Voicings from Processor. Close dialog and then retry." , value = ""})
    end
	
	--refresh the list for next launch in the event that the data has changed on the TV
    get_voicings()

    return list
	
end

function get_voicings()
    LogTrace ("get_voicings()")
    local command = "VOILIST?"
    PackAndQueueCommand("Get Voicings", command)
end

function getSortedList(t)
	local a = {}
	for j in pairs(t) do 
		table.insert(a,j)
	end 
	table.sort(a)
    
    local list = {}

    for i = 1,table.maxn(a) do
        local id = a[i]
		table.insert (list, {text = id, value = t[id]})
    end
	
    return list
end
